/**
 * @file	chtmltbl.h
 * @brief	#CHtmlTable class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #CHtmlTable class definition.
 *   The class handles table data for XHTML data.
 */

#ifndef CHTMLTBL_H_INCLUDED
#define CHTMLTBL_H_INCLUDED

#include <Coconut/ccsv.h>
#include <LibXmlKit/LibXmlKit.h>

/**
 * @brief	CHtmlTable class operates the table data for XHTML.
 * @par Description
 *   #CHtmlTable class operates the table data for XHTML.
 */
@interface CHtmlTable : NSObject 
{
	/** Root node of XHTML table. */
	XmlNode *	table_root ;
	/** Caption node */
	XmlNode *	caption_node ;
}

  /**
   * @brief Generate XHTML table from CSV data.
   * @param csvtable Source table.
   * @param caption Caption of the table. You can pass this by giving nil.
   * @param withborder True when border line (frame) is required for the table.
   * @par Description
   *   Generate XHTML table from CSV data.
   */
  + parseCsv: (CCsv *) csvtable caption: (NSString *) caption
      withBorder: (BOOL) withborder ;

  /**
   * @brief	Initialize html table instance 
   * @return	Initialized object.
   * @par Description
   *   Initialize html table instance.
   */
  - init ;

  /**
   * @brief	Release the content of CSV data.
   * @par Description
   *   Release the content of CSV data.
   */
  - (void) dealloc ;

  /**
   * @brief	Get root node of XHTML table.
   * @return	Root node of table.
   * @par Description
   */
  - (XmlNode *) rootNode ;

  /**
   * @brief	Set caption to the table.
   * @param	caption Caption string, 
   * 		If you pass nil, the current caption is removed.
   * @par Description
   *   Set caption to the table.
   *   If you pass nil as caption, the current caption is removed.
   */
  - (void) setCaption: (NSString *) caption ;

  /**
   * @brief	Set/reset the border line for table
   * @param	flag Set true if you need border.
   * @par Description
   *    Set/reset the border line for the table.
   *    If you pass true, border is shown for HTML table.
   */
  - (void) setBorder: (BOOL) flag ;

  /**
   * @brief Generate XHTML table from CSV data.
   * @return Result status. Zero means no error.
   * @param csvtable Source table.
   * @par Description
   *   Generate XHTML table from CSV data.
   */
  - (int) parseCsv: (CCsv *) csvtable ;

@end

#endif /* CHTMLTBL_H_INCLUDED */

