/**
 * @file	clist.h
 * @brief	Definition of #CList class
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   Definition of #CList class.
 *   #CList class operates single linked data set.
 */

#ifndef CLIST_H_INCLUDED
#define CLIST_H_INCLUDED

#include <Coconut/coconut-sysdep.h>

/**
 * @brief	CList operates list of objects.
 * @par Description
 *   CList operates list of objects.
 */
@interface CList : NSObject 
{
	/** Content object */
	id		current_obj ;
	/** Pointer for next line */
	CList *		list_next ;
	/** Pointer for previous line */
	CList *		list_prev ;
}
  
  /**
   * @brief	Add new object to the tail of the list.
   * @return	1st node of linked list.
   * @param	obj Object to add to list as last item.
   * @param	list Destination list to add new object.
   * @par Description
   *   Link src2 list after src1. Check following specification:
   *   <ol>
   *     <li>If "list" is nil, The new list which contains only "obj" is
   *         generated and returned.
   *     <li>In the other case, the new item which contains "obj" is
   *         generated and added to the last of "list".
   *   </ol>
   */
  + (CList *) appendObject: (id) obj after: (CList *) list ;
  
  /**
   * @brief	Link src2 list after src1 list.
   * @return	Linked list.
   * @param	src1 Next list. The previous link of it is replaced.
   * @param	src2 Previous node. The next link of it is replaced.
   * @par Description
   *   Link src2 list after src1. Check following specification:
   *   <ul>
   *     <li>The previous link of src1 is replaced by src2.
   *     <li>The next link of src2 is replaced by src1.
   *   </ul>
   */
  + (void) link: (CList *) src1 after: (CList *) src2 ;
  
  /**
   * @brief	Initialize list.
   * @par Description
   *   Initialize list.
   * @param
   *   obj Content of the list.
   * @return
   *   The object itself.
   */
  - initList: (id) obj ;
  
  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;
  
  /**
   * @brief	Set next node.
   * @par Description
   *  Set next node.
   */
  - setNextNode: (CList *) node ;
  
  /**
   * @brief	Set previous node.
   * @par Description
   *  Set previous node.
   */
  - setPrevNode: (CList *) node ;

  /**
   * @brief	Get element object.
   * @par Description
   *   Get element object.
   */
  - (id) obj ;

  /**
   * @brief	Set element object.
   * @par Description
   *   Set element object.
   * @param obj
   *   Object to replace current one.
   */
  - set: (id) obj ;

  /**
   * @brief	Get number of data in list.
   * @return	Number of data.
   * @par Description
   *   Get number of data in list.
   */
  - (uint_t) num ;

  /**
   * @brief	Get next node.
   * @par Description
   *   Get next node.
   * @return
   *   Next node or nil.
   */
  - (CList *) next ;

  /**
   * @brief	Get previous node.
   * @par Description
   *   Get previois node.
   * @return
   *   Previous node or nil.
   */
  - (CList *) prev ;

  /**
   * @brief	Get first node of the list.
   * @par Description
   *   Get first node of the list.
   * @return
   *   First node or nil.
   */
  - (CList *) first ;

  /**
   * @brief	Get last node of the list.
   * @par Description
   *   Get last node of the list.
   * @return
   *   Last node or nil.
   */
  - (CList *) last ;

  /**
   * @brief	Add new next node and set the new element.
   * @return	New added node.
   * @param	obj New element to add.
   * @par Description
   *  Add new next node and set the new element to it.
   */
  - (CList *) addObject: (id) obj ;

  /**
   * @brief	Insert new previous node and set the new element.
   * @return	New inserted node.
   * @param	obj New element to add.
   * @par Description
   *  Insert new previous node and set the new element to it.
   */
  - (CList *) insertObject: (id) obj ;

  /**
   * @brief	Append node to last and set the new element.
   * @return	New appended node.
   * @param	obj New element to add.
   * @par Description
   *  Append node to last and set the new element to it.
   */
  - (CList *) appendObject: (id) obj ;

  /**
   * @brief	Prepend node to head of the list and set the new element.
   * @return	New prepended node.
   * @param	obj New element to add.
   * @par Description
   *   Prepend node to head of the list and set the new element.
   */
  - (CList *) prependObject: (id) obj ;

@end

#endif /* CLIST_H_INCLUDED */

