/**
 * @file	copt.h
 * @brief	#COpt class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #COpt class definition.
 *   #COpt class contains information for command line option and it's 
 *   argument.
 */

#ifndef COPT_H_INCLUDED
#define COPT_H_INCLUDED

#include <Coconut/coconut-sysdep.h>

/** Type of argument of command line option */
typedef enum {
	/** Argument: none */
	COPT_ARG_TYPE_NONE,
	/** Argument: string */
	COPT_ARG_TYPE_STRING,
	/** Argument: integer */
	COPT_ARG_TYPE_INT
} COptArgType ;

/**
 * @class	COpt
 * @brief	COpt class operates the information for 
 *		command line options.
 * @par Description
 *   COpt class operates the information for command line options.
 */
@interface COpt : NSObject 
{
	/** Option ID */
	uint_t			opt_id ;

	/** is_given Flag means the option is given or not. */
	BOOL			is_given ;

	/** Long option name which started by '--' */
	NSString *	long_name ;
	/** Short option name which started by '-' */
	char		short_name ;

	/** Type of argument */
	COptArgType	arg_type ;
	/** Content of argument */
	union {
		/** String to keep str arg option */
		NSString * 	str_arg ;
		/** Integer to keep int arg option */
		int		int_arg ;
	} arg_data ;

	/** Description for the option (used by help option) */
	NSString *	opt_desc ;
	/** Description for the argument (used by help option) */
	NSString *	arg_desc ;
}

  /**
   * @brief	Initialize COpt instance for no arg option.
   * @return	The object itself.
   * @param	sname Short name (character).
   * @param	lname Long name (string).
   * @param	odesc Description for option.
   * @par Description
   *   Initialize COpt instance for no arg option.
   */
  - initOptNoArg: (char) sname longName: (const char *) lname
      optDesc: (const char *) odesc ;
  
  /**
   * @brief	Initialize COpt instance which has string argument.
   * @return	The object itself.
   * @param	sname Short name (character).
   * @param	lname Long name (string).
   * @param	odesc Description for option.
   * @param	adesc Description for argument.
   * @par Description
   *   Initialize COpt instance which has string argument.
   */
  - initOptHasStr: (char) sname longName: (const char *) lname
      optDesc: (const char *) odesc argDesc: (const char *) adesc ;
  
  /**
   * @brief	Initialize COpt instance which has number argument.
   * @return	The object itself.
   * @param	sname Short name (character).
   * @param	lname Long name (string).
   * @param	odesc Description for option.
   * @param	adesc Description for argument.
   * @par Description
   *   Initialize COpt instance which has number argument.
   */
  - initOptHasNum: (char) sname longName: (const char *) lname
      optDesc: (const char *) odesc argDesc: (const char *) adesc ;
  
  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;

  /**
   * @brief	Get option id.
   * @return	Option id.
   * @par Description
   *   Get option id. The value is given automatically.
   */
  - (int) optId ;

  /**
   * @brief	Set "is_given" flag in object.
   * @param	flag New value for the flag.
   * @par Description
   *    Set "is_given" flag in object.
   */
  - (void) setGivenFlag: (BOOL) flag ;

  /**
   * @brief	Return true when this option is found in command line.
   * @return	True when this option is found in command line.
   * @par Description
   *   True when this option is found in command line.
   */
  - (BOOL) isGiven ;

  /**
   * @brief	Get long name of the option.
   * @return	Long name string.
   *		If the long name is not set, this value will be nil.
   * @par Description
   *   Get long name of the option.
   */
  - (NSString *) longName ;

  /**
   * @brief	Get short name of the option.
   * @return	Short name string.
   *		If the short name is not set, this value will be 0.
   * @par Description
   *   Get short name of the option.
   */
  - (char) shortName ;

  /**
   * @brief	Get type of argument which is required by this option.
   * @return	Type of argument.
   * @par Description
   *   Get type of argument which is required by this option.
   */
  - (COptArgType) argType ;

  /**
   * @brief	Set string as an argument.
   * @param	src Source string.
   * @par Description
   *    Set string as an argument.
   */
  - (void) setStrArg: (NSString *) src ;

  /**
   * @brief	Get argument of string option.
   * @return 	String argument.
   * @par Description
   *   Get argument of string option. 
   *   If this object does not allow to have string argument.
   *   This method caused error.
   */
  - (NSString *) strArg ;

  /**
   * @brief	Set integer as an argument.
   * @param	src Source value.
   * @par Description
   *   Set value of integer argument.
   */
  - (void) setIntArg: (int) src ;

  /**
   * @brief	Get argument of integer option.
   * @par Description
   *   Get argument of integer option. 
   *   If this object does not allow to have integer argument.
   *   This method caused error.
   */
  - (int) intArg ;

  /**
   * @brief	Get descriotion string for option.
   * @return	Description string.
   * @par Description
   *   Get descriotion string for option.
   */
  - (NSString *) optDescription ;

  /**
   * @brief	Make help message used for left section.
   * @return	Left section of help message.
   * @par Description
   *    Make help message used for left section.
   *	It contains short name and long name. 
   */
  - (NSString *) stringForOptionsForHelpMessage ;

  /**
   * @brief	Print the content of option data.
   * @param	outfp Output stream.
   * @par Description
   *   Print the content of option data.
   */
  - (void) print: (FILE *) outfp ;

@end

#endif /* COPT_H_INCLUDED */

