/**
 * @file	coptpsr.h
 * @brief	#COptPsr class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #COptPsr class definition.
 *   #COptPsr parses command line arguments and
 *   set result to #COpt objects in it.
 */

#ifndef COPTPSR_H_INCLUDED
#define COPTPSR_H_INCLUDED

#include <Coconut/copt.h>
#include <Coconut/clist.h>

/**
 * @class	COptPsr
 * @brief	COptPsr class parses command line options and arguments.
 * @par Description
 *   COptPst class parses command line options and arguments.
 */
@interface COptPsr : NSObject 
{
	/** List of options given by addOpt() method */
	CList *		opt_list ;
	/** List of arguments. This is result of parsing. */
	CList *		arg_list ;
	/** Name of application. It is given after parse() method */
	NSString *	app_name ;

	/** "help" option */
	COpt *		opt_help ;
	/** "usage" option */
	COpt *		opt_usage ;
}

  /**
   * @brief	Initialize instance of COptPsr class.
   * @return	The object itself.
   * @par Description
   *   Initialize instance of COptPsr class.
   */
  - init ;
  
  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;
 
  /**
   * @brief	Get application name.
   * @return	Name of application
   * @par Description
   *   Get application name.
   * @warning
   *   This result will be valid after calling parse() method.
   */
  - (NSString *) appName ;

  /**
   * @brief	Add option to parser.
   * @param	opt Source option to add.
   * @par Description
   *   Add option to parser.
   */
  - (void) addOpt: (COpt *) opt ;

  /**
   * @brief	Add '--help' option to parser.
   * @par Description
   *   Add '--help' option to parser.
   */
  - (void) addHelpOpt ;

  /**
   * @brief	Add '--usage' option to parser.
   * @par Description
   *   Add '--usage' option to parser.
   */
  - (void) addUsageOpt ;

  /**
   * @brief	Get number of options.
   * @return	Number of options in the parser.
   * @par Description
   *   Get number of options.
   */
  - (uint_t) optNum ;

  /**
   * @brief	Start parsing the command line.
   * @retval	true Parsing finished without error.
   * @retval	false Some error found in command line.
   * @param	argc Number of arguments.
   * @param	argv Array of argument strings.
   * @param	withmsg If this value is true, error messages are printed.
   * @par Description
   *   Start parsing the command line.
   */
  - (BOOL) parse: (int) argc and: (const char **) argv 
    withErrorMessage: (BOOL) withmsg ;

  /**
   * @brief	Get list of arguments of command line.
   * @return	List of arguments. It contains CString data.
   *            This pointer will NOT be nil.
   * @par Description
   *   Get list of arguments of command line. The data class is CSting.
   */
  - (CList *) argList ;

@end

#endif /* COPTPSR_H_INCLUDED */

