/**
 * @file	cscalar.h
 * @brief	#CScalar class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *    #CScalar class definition.
 *    This class operates any typed data.
 */

#ifndef CSCALAR_H_INCLUDED
#define CSCALAR_H_INCLUDED

#include <Coconut/coconut-sysdep.h>

/** Type of content data of #CScalar object. */
typedef enum {
	CSCALAR_TYPE_UNKNOWN,
	CSCALAR_TYPE_INT,
	CSCALAR_TYPE_DOUBLE,
	CSCALAR_TYPE_STR,
	CSCALAR_TYPE_PTR,
	CSCALAR_TYPE_OBJ
} CScalarType ;

/**
 * @brief	CScalar operates list of objects.
 * @par Description
 *   CScalar operates list of objects.
 */
@interface CScalar : NSObject 
{
	/** Type of scalar data */
	CScalarType	scalar_type ;
	/** Union for all scalar data */
	union {
		/** Integer data */
		int		v_int ;
		/** Double data */
		double		v_double ;
		/** String data */
		NSString *	v_str ;
		/** Pointer data */
		void *		v_ptr ;
		/** Object data */
		id		v_obj ;
	} content ;
}

  /**
   * @brief	Initialize scalar object.
   * @par Description
   *   Initialize scalar object.
   *   The initial value is integer zero.
   * @return
   *   The object itself.
   */
  - init ;
  
  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;

  /**
   * @brief	Release the content data.
   * @return	Self pointer.
   * @par Description
   *   Release the content data. The type will be int and value will be 0.
   */
  - clear ;

  /**
   * @brief	Set integer value.
   * @param	src Source int.
   * @par Description
   *   Set integer value.
   */
  - setIntVal: (int) src ;

  /**
   * @brief	Get int value of the content data.
   * @return	Int value.
   * @par Description
   *   Get int value of the content data.
   *   If the content data is not int, this method returns 0.
   */
  - (int) intVal ;

  /**
   * @brief	Set double value.
   * @param	src Double value.
   * @par Description
   *   Set double value.
   */
  - setDoubleVal: (double) src ;

  /**
   * @brief	Get double value of the content data.
   * @return	Double value.
   * @par Description
   *   Get double value of the content data.
   *   If the content data is not double, this method returns 0.0.
   */
  - (double) doubleVal ;

  /**
   * @brief	Set string.
   * @param	src Source string.
   * @par Description
   *   Set string to the object.
   */
  - (void) setStr: (NSString *) src ;

  /**
   * @brief	Get string of the content data.
   * @return	String.
   * @par Description
   *   Get string of the content data.
   *   If the content data is not string, this method returns null.
   */
  - (NSString *) str ;

  /**
   * @brief	Set pointer.
   * @param	ptr Source pointer. You can pass null.
   * @param 	size Size of the data. If you pass null as ptr,
   *	  	this value is forced to be zero.
   * @par Description
   *   Set pointer to the object.
   */
  - (void) setPtr: (void *) ptr size: (size_t) size ;

  /**
   * @brief	Get pointer value of the content data.
   * @return	Poiter of content data.
   * @par Description
   *   Get pointer of the content data.
   *   If the content data is not pointer, this method returns null.
   */
  - (void *) ptr ;

  /**
   * @brief	Set object.
   * @return	Self pointer.
   * @par Description
   *   Set object to the object.
   */
  - setObj: (id) obj ;

  /**
   * @brief	Get object of the content data.
   * @return	Content object.
   * @par Description
   *   Get object of the content data.
   *   If the content data is not object, this method returns nil.
   */
  - (id) obj ;

  /**
   * @brief	Print the content of the data.
   * @param	fp Outut stream.
   * @par Description
   *   Print the content of data.
   */
  - debugPrint: (FILE *) fp ;
@end

#endif /* CSCALAR_H_INCLUDED */

