/**
 * @file	cstack.h
 * @brief	Definition of #CStack class
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   Definition of #CStack class.
 *   #CList class operates single linked data set.
 */

#ifndef CSTACK_H_INCLUDED
#define CSTACK_H_INCLUDED

#include <Coconut/coconut-sysdep.h>

/** Pseudo type of stack element. The real type is defined in cstack.m */
typedef struct _CStackElm	CStackElm ;
/** Pointer of #CStackElm */
typedef CStackElm *		CStackElmPtr ;
/** Const pointer of #CStackElm */
typedef const CStackElm *	CConstStackElmPtr ;

/**
 * CStack operates stack of objects.
 */
@interface CStack : NSObject 
{
	/** Pointer of the stack. The lower address contains deeper 
	    data in the stack. */
	CStackElmPtr	stack_bottom ;
	/** Number of valid objects in the stack */
	uint_t		current_num ;
	/** Maximum size of stack */
	uint_t		max_num ;
}

/**
 * @brief	Initialize stack.
 * @par Description
 *   Initialize stack.
 */
- init ;

/**
 * @brief	Deallocate stack.
 * @par Description
 *   Deallocate stack.
 */
- (void) dealloc ;

/**
 * @brief	Push object to stack.
 * @param	obj New object to stack.
 * @par Description
 *   Deallocate stack.
 */
- (void) push: (id) obj ;

/**
 * @brief	Pop data from stack.
 * @retval	obj Object at top of stack.
 * @retval	nil When the stack is empty.
 * @par Description
 *    Pop data from stack.
 */
- (id) pop ;

/**
 * @brief	Get number of objects in the stack.
 * @return	Number of objects.
 * @par Description
 *   Get number of objects in the stack.
 */
- (uint_t) num ;

/**
 * @brief	Get object at the top of stack.
 * @retval	obj Object at top of stack.
 * @retval	nil Stack is empty.
 * @par Description
 *   Get object at the top of stack. The object is not poped.
 */
- (id) objectAtTop ;

/**
 * @brief	Get object by index.
 * @retval	obj Object at index depth of stack.
 * @retval	nil Object is not found there.
 * @par Description
 *   Get object by index.
 */
- (id) objectAtIndex: (uint_t) idx ;

@end

#endif /* CSTACK_H_INCLUDED */

