/**
 * @file	ctext.h
 * @brief	#CText class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *    #CText class definition.
 *    This class operates any typed data.
 */

#ifndef CTEXT_H_INCLUDED
#define CTEXT_H_INCLUDED

#include <Coconut/coconut-sysdep.h>

/**
 * @brief	CText operates list of objects.
 * @par Description
 *   CText operates list of objects.
 */
@interface CText : NSObject 
{
	/** Array of lines in text */
	NSMutableArray *	line_array ;
}

  /**
   * @brief	Initialize #CText object.
   * @par Description
   *   Initialize #CText object.
   * @return
   *   The object itself.
   */
  - init ;
 
  /**
   * @brief	Initialize text object by string.
   * @return	New text.
   * @param	str Source string.
   * @par Description
   *    Initialize text object by string.
   */
  - initWithString: (NSString *) str ;

  /**
   * @brief	Release resource for the object.
   * @par Description
   *   Release resource for the object.
   */
  - (void) dealloc ;

  /**
   * @brief	Get number of lines in the text.
   * @return	Number of lines.
   * @par Description
   *   Get number of lines in the text.
   */
  - (uint_t) lineNum ;

  /**
   * @brief	Get one line from text.
   * @retval	obj The string for the line.
   * @retval	nil The indexed object is not found.
   * @param	index Index to choose a line.
   * @par Description
   *   Get one line from text.
   */
  - (NSString *) lineAtIndex: (uint_t) index ;

  /**
   * @brief	Append new line to the text.
   * @param	src Source line.
   * @par Description
   *   Append new line to the text.
   *   The source line must <b>NOT</b> have newline code in it.
   *   (See appendSomeLines() method).
   * @warning
   *   Do not give the line which contains newline code.
   *   This method does not split the line into some lines.
   */
  - (void) appendLine: (NSString *) src ;

  /**
   * @brief	Append some lines to the text
   * @param	src Source string which contans some words and newlines.
   * @par Description
   *   Append new lines to the text.
   *   The source string can contain some newlines.
   *   (See appendLine() method).
   */
  - (void) appendSomeLines: (NSString *) src ;

  /**
   * @brief	Prepend new string as a 1st line in text.
   * @param	src Source line. The string must <b>NOT</b> contain
   *		newline codes.
   * @par Description
   *    Prepend new string as a 1st line in text.
   *   The source line must <b>NOT</b> have newline code in it.
   *   (See prependSomeLines() method).
   * @warning
   *   Do not give the line which contains newline code.
   *   This method does not split the line into some lines.
   */
  - (void) prependLine: (NSString *) src ;

  /**
   * @brief	Prepend some lines to the heading of text
   * @param	src Source string which contans some words and newlines.
   * @par Description
   *   Prepend new lines to the heading of text.
   *   The source string can contain some newlines.
   *   (See prependLine() method).
   */
  - (void) prependSomeLines: (NSString *) src ;

  /**
   * @brief	Get max width of line in the text.
   * @return	Width of text.
   * @par Description
   *   Get max width of line in the text.
   */
  - (uint_t) width ;

  /**
   * @brief	Print the content of text.
   * @param	outfp Output file stream.
   * @par Description
   *    Print the content of text.
   */
  - (void) print: (FILE *) outfp ;

  /**
   * @brief	Join other text for each lines and generate text from them.
   * @param	text Source text to paste.
   * @par Description
   *    Paste other text for each lines.
   *	Each lines are joined and the indentation of right line
   *    are defined by maximum width of left lines.
   */
  - (CText *) paste: (CText *) text ;

@end

#endif /* CTEXT_H_INCLUDED */

